﻿package com.ophone.dazhihui.http;

import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;

import com.ophone.dazhihui.http.QueryApList.Carriers;

import android.content.Context;
import android.database.Cursor;
import android.net.ConnectivityManager;

/**
 * This class is to help connecting to one AP.
 * 
 * 用来简化网络接入点初始化和网络连接的处理。
 */
public class ConnectByAp {
	
	public ConnectByAp(Context context) {
		mContext = context;
		mConnMgr = (ConnectivityManager) mContext.getSystemService(Context.CONNECTIVITY_SERVICE);

//        mServiceHandler = new ServiceHandler(Looper.myLooper(), mContext);
	}


	/**
	 * Connect to one AP by the AP's type name.
	 * In the latest version of OPhone, the AP type list(AP group) comes from manufactory, 
	 * and it can't be configured by the phone user.
	 * But each type of AP group may have several AP items configured by phone user.
	 * When connecting, different AP items in the given type of AP group will tried
	 * one by one, until connection is successfully build.
	 * 
	 * 通过介入点（AP）的类型连接网络。
	 * 在最新版本的OPhone中，AP按照类型分组，每个分组一个类型名称。
	 * 分组信息在手机出厂时固定，不可以由用户配置。
	 * 在每个AP分组下，可以由用户配置若干AP条目。
	 * 在发起连接时，指定AP分组下的所有AP条目会被逐个尝试，直到连接成功。
	 * 被调用时，该方法会被一直阻滞，直到连接结束（成功或者失败）。
	 * 
	 * @param apType AP type name of a certain AP group
	 * @param millisec Connecting timeout in milliseconds, it should be great than 0, or no timeout property will be set.
	 * @return	The connecting result in boolean type
	 */
	public boolean connect(String apType, long millisec) {
		if (apType == null)
			return false;

        int result = mConnMgr.startUsingNetworkFeature(
                ConnectivityManager.TYPE_MOBILE, apType);

        
        return result != -1;
	}
	
	/**
	 * Connect to one AP by the AP's ID.
	 * In the latest version of OPhone, the AP type list(AP group) comes from manufactory, 
	 * and it can't be configured by the phone user.
	 * But each type of AP group may have several AP items configured by phone user.
	 * When connecting, different AP items in the given type of AP group will tried
	 * one by one, until connection is successfully build.
	 * 
	 * 通过介入点（AP）的ID连接网络。
	 * 在最新版本的OPhone中，AP按照类型分组，每个分组一个类型名称。
	 * 分组信息在手机出厂时固定，不可以由用户配置。
	 * 在每个AP分组下，可以由用户配置若干AP条目。
	 * 在发起连接时，指定AP分组下的所有AP条目会被逐个尝试，直到连接成功。
	 * 被调用时，该方法会被一直阻滞，直到连接结束（成功或者失败）。
	 * 
	 * @param apType AP ID of a AP item in a certain AP group
	 * @param millisec Connecting timeout in milliseconds, it should be great than 0, or no timeout property will be set.
	 * @return	The connecting result in boolean type
	 */
	public boolean connect(int apId, long millisec) {
		if (apId == -1)
			return false;
		
		Cursor c = mContext.getContentResolver().query(Carriers.CONTENT_URI, 
				new String[] {Carriers.TYPE}, String.format("%s=\"%s\"", Carriers._ID, apId), null, null);
		
        if (c == null)
        	return false;

        String apType = null;
        if (c.moveToNext())
        	apType = c.getString(0);

        if(c!=null){
        	try{
        		c.close();
        	}catch(Exception e){
        		
        	}
        }
        return connect(apType, millisec);
	}
	
	/**
	 * Called when application not use network anymore
	 * Notice: only called when application is quiting
	 */
	public void disconnect() {
//		mServiceHandler.disconnect();
//		mServiceHandler = null;
	}

	
	
	/*
	 * <Sample Code>
	 * Handle Network Status Changed Message.
	 */
//	public class ServiceHandler extends Handler {
//	    public ServiceHandler(Looper looper, Context context) {
//	        super(looper);
//
//	        mConnectivityListener = new NetworkConnectivityListener(context);
//	        mConnectivityListener.setServiceHandler(this, EVENT_DATA_STATE_CHANGED);
//	        mConnectivityListener.startListening();
//	    }
//
//	    private static final int EVENT_DATA_STATE_CHANGED = 1;
//
//	    private NetworkConnectivityListener mConnectivityListener;
//	    
//	    /**
//	     * Handle incoming Network Status Changed Message
//	     */
//	    @Override
//	    public void handleMessage(Message msg) {
//	    	//!Log.d(TAG, "handle message: msg.what=" + msg.what);
//	    	
//	        switch (msg.what) {
//	            case EVENT_DATA_STATE_CHANGED: {
////	                printlnStatus("--------------------------------");
////	                printlnStatus("!!!Network status changed.");
//	            	//!Log.d(TAG, "Network status changed");
//	                
//	                /*
//	                 * If we are being informed that connectivity has been
//	                 * established, you can start socket to send/receive data.
//	                 */
//	                if (mConnectivityListener == null)
//	                    return;
//
//	                NetworkInfo info = mConnectivityListener.getNetworkInfo();
//	                
//	                if (info != null) {
//	                    /*
//	                     * Firstly, you can get the AP type of the Network Info.
//	                     * And check the type, if it is not the one that you care,
//	                     * ignore the message.
//	                     */
//	                    String apType = info.getApType();
//
//	                    NetworkInfo.State state = info.getState();
//	                    if (state == NetworkInfo.State.CONNECTED) {
//	                        /*
//	                         * Check the reason for detailed info.
//	                         * Please refer to Phone.REASON_*
//	                         */
//	                    	// String reason = info.getReason();
//
//	                        if (TextUtils.equals(DataConnectivityConstants.FEATURE_ENABLE_INTERNET, apType))
//	                        	mNetInterface = "";
//	                        else {
//	                            /*
//	                             * Don't need bind socket to Network Interface for Internet AP Type.
//	                             * But if you used special AP type before, you should clean the interface name
//	                             * <NOTE>
//	                             * Special AP type, such as: wap, need bind socket to Network Interface.  
//	                             */
//	                        	mNetInterface = info.getInterfaceName();
//	                        }
//	                    } else if (state == NetworkInfo.State.DISCONNECTED) {
//
//	                        /*
//	                         * Network disconnected. Should check the detailed state and reason.
//	                         */
//
//	                        NetworkInfo.DetailedState detailState = info.getDetailedState();
//	                        String reason = info.getReason();
//	                        if (detailState == NetworkInfo.DetailedState.FAILED) {
//	                            /* 
//	                             * Failed to setup data connection
//	                             * Retry, or show error information to user
//	                             */
////	                            printlnStatus("!!!FAILED to connect.");
//	                        } else if (TextUtils.equals(reason, DataConnectivityConstants.REASON_DEACTIVED_BY_NETWORK)) {
//	                            /*
//	                             * Connection was broken by network.
//	                             * Retry, or show error information to user
//	                             */
////	                            printlnStatus("!!!Connection was broken by network.");
//	                        } else if (TextUtils.equals(reason, DataConnectivityConstants.REASON_ROAMING_ON)) {
//	                            /*
//	                             * Connection was broken for leaving the home network and 
//	                             * DATA_ON_ROMAING option is disabled.
//	                             */
////	                            printlnStatus("!!!Connection was broken for roaming.");
//	                        } else {
//	                            /*
//	                             * For other state and reason, you should omit them in most cases.  
//	                             */
////	                            Log.d(TAG, "Network status changed, detailState: " + detailState + ", reason: " + reason);
//	                        }
//	                    } else {
//	                        /*
//	                         * Log the network status
//	                         */
////	                        Log.d(TAG, "Network status changed, current state: " + state);
//	                    }
//	                }
//	                break;
//	            }
//	            default:
////	                Log.w(TAG, "what=" + msg.what);
//	        }
//
//        	notifyState();
//	    }
//	    
//		public void disconnect() {
//			mConnectivityListener.stopListening();
//		}
//	}


	private void waitNotification(long millisec) {
		int count = 1;
		final int try_count = 100;
		final long try_delay = (millisec <= 0) ? 300 : (millisec <= try_count) ? 1 : millisec / try_count;
		
		while (count < try_count) {
			//!Log.d(TAG, "try wait: " + try_delay + "ms/" + millisec + "ms");
    		if (waitState(try_delay))
    			break;
    		if (millisec > 0)
    			count++;
		}
	}


	private Semaphore semaphore = new Semaphore(0);
	public boolean waitState(long millisec) {
		try {
			if (millisec > 0)
				return semaphore.tryAcquire(millisec, TimeUnit.MILLISECONDS);
			else {
        		semaphore.acquire();
        		return true;
        	}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		return false;
	}

	public void notifyState() {
		semaphore.release();
	}

	private static final String TAG = ConnectByAp.class.getSimpleName();
	
	// the context
	private Context mContext;
	// network state listener
//	private ServiceHandler mServiceHandler;

	/**
	 * connect manager
	 */
	public ConnectivityManager mConnMgr;

    public String mNetInterface;

}
