package com.ophone.dazhihui.http;

import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

/**
 * 
 * @author OPhone SDK Group, CMCC
 *
 */
public class QueryApList {

	/**
	 * Constructor of QueryApList
	 * @param context	the context of the application
	 */
	public QueryApList(Context context) {
		this.mContext = context;
		mListApn = new String[]{"internet","wap","cmmail","mms"};
		mListProxy = new String[]{null,"10.0.0.172",null,null};
		mListProxyPort= new int[]{0,80,0,0};
//		loadApListInfo();
	}

	/**
	 * Get AP list's size.
	 * @return the total number of all APs having been configured out.
	 */
	public int getApSize() {
		return (mListTitle == null) ? 0 : mListTitle.length;
	}
	
	/**
	 * Get AP ID at the given index.
	 * @param index
	 * @return the Id . If no any AP exists, then it is -1.
	 */
	public int getApId(int index) {
		if (mListApId == null || index >= mListApId.length)
			return -1;
		return mListApId[index];
	}

	/**
	 * Get AP title at the given index.
	 * @param index
	 * @return the title String. If no any AP exists, then it is null.
	 */
	public String getApTitle(int index) {
		if (mListTitle == null || index >= mListTitle.length)
			return null;
		return mListTitle[index];
	}

	/**
	 * Get AP value at the given index.
	 * @param index
	 * @return the value String. If no any AP exists, then it is null.
	 */
	public String getApValue(int index) {
		if (mListApn == null || index >= mListApn.length)
			return null;
		return mListApn[index];
	}


	/**
	 * Get AP type at the given index.
	 * @param index
	 * @return the type String. If no any AP exists, then it is null.
	 */
	public String getApType(int index) {
		if (mListType == null || index >= mListType.length)
			return null;
		return mListType[index];
	}
	
	/**
	 * Get AP proxy host name at the given index.
	 * @param index
	 * @return the proxy host name. If no any AP exists, then it is null.
	 */
	public String getApProxy(int index) {
		if (mListProxy == null || index >= mListProxy.length)
			return null;
		return mListProxy[index];
	}

	/**
	 * Get AP proxy port at the given index.
	 * @param index
	 * @return the proxy port number. If no any AP exists, then it is -1.
	 */
	public int getApProxyPort(int index) {
		if (mListProxyPort == null || index >= mListProxyPort.length)
			return -1;
		return mListProxyPort[index];
	}

	/**
	 * Get AP ID.
	 * @return the id array. If no any AP exists, then it is null.
	 */
	public int[] getApId() {
		return mListApId;
	}

	/**
	 * Get AP title.
	 * @return the title String array. If no any AP exists, then it is null.
	 */
	public String[] getApTitle() {
		return mListTitle;
	}

	/**
	 * Get AP value
	 * @return the value String array
	 */
	public String[] getApValue() {
		return mListApn;
	}


	/**
	 * Get AP type
	 * @return the type String array
	 */
	public String[] getApType() {
		return mListType;
	}

	
	/*
	 * Load AP list
	 */
	protected void loadApListInfo() {
		Cursor c = mContext.getContentResolver().query(Carriers.CONTENT_URI, 
				new String[] {Carriers._ID, Carriers.NAME, Carriers.APN, Carriers.TYPE, Carriers.PROXY, Carriers.PORT}, 
				null, null, Carriers.TYPE);
		
        if (c == null)
        	return;

        int count = c.getCount();
        mListApId = new int[count];
    	mListTitle = new String[count];
    	mListApn = new String[count];
    	mListType = new String[count];
    	mListProxy = new String[count];
    	mListProxyPort = new int[count];
        try {
        	int i = 0;
            while (c.moveToNext()) {
            	mListApId[i] = c.getInt(0);
            	mListTitle[i] = c.getString(1);
            	mListApn[i] = c.getString(2);
            	mListType[i] = c.getString(3);
            	mListProxy[i] = c.getString(4);
            	mListProxyPort[i] = c.getInt(5);
               // Log.d("TEST", mListTitle[i]+"; "+mListApn[i] + "; " + mListType[i]);
                i++;
            }
        } finally {
            if(c!=null){
            	try{
            		c.close();
            	}catch(Exception e){
            		
            	}
            }
        }
	}
	
	
	protected static final class Carriers {
        /**
         * The content:// style URL for this table
         */
        public static final Uri CONTENT_URI =
            Uri.parse("content://telephony/carriers");

        /**
         * The default sort order for this table
         */
        public static final String DEFAULT_SORT_ORDER = "name ASC";

        public static final String _ID = "_id";
        
        public static final String NAME = "name";

        public static final String APN = "apn";

        public static final String PROXY = "proxy";

        public static final String PORT = "port";

        public static final String MMSPROXY = "mmsproxy";

        public static final String MMSPORT = "mmsport";

        public static final String SERVER = "server";

        public static final String USER = "user";

        public static final String PASSWORD = "password";

        public static final String MMSC = "mmsc";

        public static final String MCC = "mcc";

        public static final String MNC = "mnc";

        public static final String NUMERIC = "numeric";

        public static final String TYPE = "type";

        public static final String CURRENT = "current";
        
        public static final String VISIBLE = "visible";
        
        public static final String PRELOADED = "preloaded";
        
    }

    /*
     * Keeps Context of the running Application
     * It is initialized in constructor.
     */
    protected Context mContext = null;
    
    /*
     * AP item's id.
     */
    protected int[] mListApId = null;
    
    /*
     * AP item's title.
     */
    protected String[] mListTitle = null;

    /*
     * AP value.
     */
    protected String[] mListApn = null;

    /*
     * AP type.
     */
    protected String[] mListType = null;

    /**
     * AP Proxy host
     */
    protected String[] mListProxy = null;

    /**
     * AP Proxy Port
     */
    protected int[] mListProxyPort = null;
}

